import json
import argparse
from text_rank import DEMO_LOC
from text_rank.graph import sentence_graph, keyword_graph
from text_rank.graph import sentence_graph_list, keyword_graph_list
from text_rank.text_rank import text_rank, text_rank_list

parser = argparse.ArgumentParser("Text Rank")
parser.add_argument("--iters", "-i", type=int, default=40)
parser.add_argument("--sents", "-s", type=int, default=3)
parser.add_argument("--words", "-w", type=int, default=5)
args = parser.parse_args()


sentences = json.load(open(DEMO_LOC / "Automatic_Summarization-sents.json"))
V, g = sentence_graph(sentences)
sents1 = text_rank(V, g, args.iters)
print([' '.join(x[0]) for x in sents1[:args.sents]])

V = sentence_graph_list(sentences)
sents2 = text_rank_list(V, args.iters)
print([' '.join(x[0].value) for x in sents2[:args.sents]])


tokens = json.load(open(DEMO_LOC / "Automatic_Summarization-tokens.json"))
V, g = keyword_graph(tokens)
keywords1 = text_rank(V, g, args.iters)
print([x[0] for x in keywords1[:args.words]])

V = keyword_graph_list(tokens)
keywords2 = text_rank_list(V, args.iters)
print([x[0].value for x in keywords2[:args.words]])



import random
import numpy as np
from tqdm import tqdm


# def sparse_textrank(vertices, graph, niter=200, dampaning=0.85):
#     ws = random(len(vertices), 1, 1)
#     print(ws.shape)
#     denorm = graph.sum(axis=1)
#     print(denorm.shape)
#     for _ in tqdm(range(niter)):
#         update = (graph / denorm).multiply(ws).sum(axis=0)
#         print(update.shape)
#         ws = (1 - dampening) + update.multiply(dampening)
#         print(ws.shape)
#     return sorted(zip(vertices, ws), key=lambda x: x[1], reverse=True)

def sum_edges(edges):
    return sum(edges.values())

def accum(vertices, i, denorm, ws):
    acc = 0
    for edge in vertices[i].edges_in:
        j = vertices[edge]
        edge_ji = j.edges_out.get(i)
        if edge_ji is not None:
            acc += edge_ji / denorm[edge] * ws[edge]
    return acc

def text_rank_list(vertices, niter=200, dampening=0.85, quiet=True):
    denorm = {}
    ws = []
    for i, vertex in enumerate(vertices):
        denorm[i] = sum_edges(vertex.edges_out)
        ws.append(random.random())

    for _ in tqdm(range(niter), disable=quiet):
        updates = {}
        for i  in range(len(vertices)):
            acc = accum(vertices, i, denorm, ws)
            updates[i] = acc
        for i, update in updates.items():
            ws[i] = (1 - dampening) + dampening * update
    return sorted(zip(vertices, ws), key=lambda x: x[1], reverse=True)


def text_rank(vertices, graph, niter=200, dampening=0.85, quiet=True):
    ws = np.random.rand(len(vertices), 1)
    denorm = np.reshape(np.sum(graph, axis=1), (-1, 1))
    for _ in tqdm(range(niter), disable=quiet):
        update = np.sum(graph / denorm * ws, axis=0)
        ws = np.reshape((1 - dampening) + dampening * update, (-1, 1))
    ws = np.reshape(ws, (-1,))
    return sorted(zip(vertices, ws), key=lambda x: x[1], reverse=True)



from setuptools import setup, find_packages

version = "0.1.1"

setup(
    name="text_rank",
    version=version,
    description="Text Rank with Cython",
    author="Brian Lester",
    author_email="blester125@gmail.com",
    url="https://github.com/blester125/text_rank",
    download_url=f"https://github.com/blester125/text_rank/archive/{version}.tar.gz",
    license="MIT",
    packages=find_packages(),
    package_data={
        'text_rank': [
            'text_rank/data/Automatic_Summarization-sents.json',
            'text_rank/data/Automatic_Summarization-tokens.json',
        ],
    },
    include_package_data=True,
    install_requires=[
        'numpy',
        'tqdm',
    ],
    keywords=['NLP']
)



import re
from math import log
from itertools import combinations
import numpy as np
#from scipy.sparse import lil_matrix, random


class Vertex:
    def __init__(self, value, index):
        self.value = value
        self.index = index
        self.edges_out = {}
        self.edges_in = {}


def filter_pos(token):
    if (
        not re.match(r"^[NJ]", token['pos']) and
        token['pos'] != 'ADJ' and
        token['pos'] != 'CD'
    ):
        return False
    return True


def overlap(s1, s2):
    intersection = len(set(s1) & set(s2))
    norm = log(len(s1)) + log(len(s2))
    return intersection / norm


def keyword_graph(tokens, winsz=2):
    vertices = []
    vocab = {}

    tokens = list(filter(filter_pos, tokens))
    for i in range(len(tokens)):
        token = tokens[i]
        if token['term'] in vocab:
            idx = vocab[token['term']]
        else:
            idx = len(vocab)
            vocab[token['term']] = idx
        if idx >= len(vertices):
            vertices.append(token['term'])

    graph = np.zeros((len(vertices), len(vertices)))
    for i in range(len(tokens)):
        token = tokens[i]
        source_idx = vocab[token['term']]
        source_node = vertices[source_idx]
        min_ = max(0, i - winsz)
        max_ = min(len(tokens) - 1, i + winsz)
        for j in range(min_, max_):
            other = tokens[j]
            if i == j:
                continue
            target_idx = vocab[other['term']]
            graph[source_idx, target_idx] = 1
            graph[target_idx, source_idx] = 1
    return vertices, graph


def keyword_graph_list(tokens, winsz=2):
    vertices = []
    vocab = {}

    tokens = list(filter(filter_pos, tokens))
    for i in range(len(tokens)):
        token = tokens[i]
        if token['term'] in vocab:
            idx = vocab[token['term']]
        else:
            idx = len(vocab)
            vocab[token['term']] = idx
        if idx >= len(vertices):
            vertices.append(Vertex(token['term'], idx))

    for i in range(len(tokens)):
        token = tokens[i]
        source_idx = vocab[token['term']]
        source_node = vertices[source_idx]
        min_ = max(0, i - winsz)
        max_ = min(len(tokens) - 1, i + winsz)
        for j in range(min_, max_):
            other = tokens[j]
            if i == j:
                continue
            target_idx = vocab[other['term']]
            target_node = vertices[target_idx]
            source_node.edges_out[target_idx] = 1
            source_node.edges_in[target_idx] = 1
            target_node.edges_out[source_idx] = 1
            target_node.edges_in[source_idx] = 1
    return vertices


def sentence_graph(sentences, sim=overlap):
    vertices = [None] * len(sentences)
    graph = np.zeros((len(sentences), len(sentences)))
    for i, j in combinations(range(len(sentences)), 2):
        vertices[i] = sentences[i]
        vertices[j] = sentences[j]
        graph[i, j] = sim(sentences[i], sentences[j])
        graph[j, i] = sim(sentences[j], sentences[i])
    return vertices, graph


def sentence_graph_list(sentences, sim=overlap):
    vertices = [None] * len(sentences)
    for i, j in combinations(range(len(sentences)), 2):
        score_ij = sim(sentences[i], sentences[j])
        score_ji = sim(sentences[j], sentences[i])
        vertices[i] = vertices[i] or Vertex(sentences[i], i)
        vertices[j] = vertices[j] or Vertex(sentences[j], j)
        vertices[i].edges_out[j] = score_ji
        vertices[i].edges_in[j] = score_ij
        vertices[j].edges_out[i] = score_ji
        vertices[j].edges_in[i] = score_ij
    return vertices


# def sparse_graph(sentences, sim=overlap):
#     graph = lil_matrix((len(sentences), len(sentences)), dtype=np.float64)
#     vertices = [None] * len(sentences)
#     for i, j in combinations(range(len(sentences)), 2):
#         graph[i, j] = sim(sentences[i], sentences[j])
#         graph[j, i] = sim(sentences[j], sentences[i])
#         vertices[i] = vertices[i] or Vertex(sentences[i], i)
#         vertices[j] = vertices[i] or Vertex(sentences[j], j)
#     return vertices, graph


def show(vertices):
    import networkx as nx
    import matplotlib.pyplot as plt
    from networkx.drawing.nx_agraph import graphviz_layout,to_agraph
    import pygraphviz as pgv
    G = nx.MultiDiGraph()
    for vertex in vertices.values():
        G.add_node(vertex.index)
        for edge, weight in vertex.edges_out.items():
            G.add_edge(vertex.index, edge, weight)
        for edge, weight in vertex.edges_in.items():
            G.add_edge(edge, vertex.index, weight)
    A = to_agraph(G)
    print(A)
    A.layout('dot')
    A.draw('test.png')


def print_graph(g):
    for v in g.values():
        print(f"Vertex: {v.index}")
        print("\tOutbound Edges:")
        for idx, weight in v.edges_out.items():
            print(f"\t\t{v.index}->{idx}: {weight}")
        print("\tInbound Edges:")
        for idx, weight in v.edges_in.items():
            print(f"\t\t{idx}->{v.index}: {weight}")
